/*
 * Delegates.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using System.Collections.Generic;
using System.Globalization;

#if NATIVE && WINDOWS
using System.Runtime.InteropServices;
#endif

using Eagle._Attributes;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Eagle._Components.Public.Delegates
{
#if NATIVE && UNIX
    //
    // NOTE: Used by the dynamic loader on Unix.
    //
    public delegate IntPtr dlopen(
        string fileName,
        int mode
    );

    public delegate int dlclose(
        IntPtr module
    );

    public delegate IntPtr dlsym(
        IntPtr module,
        string name
    );

    public delegate IntPtr dlerror();
#endif

    ///////////////////////////////////////////////////////////////////////////////////////////////

#if NATIVE && WINDOWS
    //
    // NOTE: Used by the Windows native stack checking code.
    //
    [ObjectId("20a6b621-590a-41f6-ad37-83dd56b2238c")]
    public delegate IntPtr NtCurrentTeb();

    //
    // NOTE: Used by the Windows native QueueUserAPC function.
    //
    [UnmanagedFunctionPointer(CallingConvention.StdCall)]
    [ObjectId("3c45a7d9-3b66-4a4e-af94-5260b51bccdd")]
    public delegate void ApcCallback(IntPtr data);

    //
    // NOTE: Used by the Windows native EnumWindows function.
    //
    [UnmanagedFunctionPointer(CallingConvention.StdCall)]
    [ObjectId("3b2d7140-039d-4d82-aca7-fecc8f52b311")]
    [return: MarshalAs(UnmanagedType.Bool)]
    public delegate bool EnumWindowCallback(IntPtr hWnd, IntPtr lParam);
#endif

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: Used by clients of the library for "free()" style
    //       native functions.
    //
    [ObjectId("a39742e2-79ea-4ba1-84f1-f8f0df2b9aca")]
    public delegate void FreeCallback(IntPtr data);

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: Used by the script binder and marshaller to convert a
    //       string to a System.Type object.
    //
    [ObjectId("95efe170-9c6d-4325-84fa-6e86c6b42449")]
    public delegate Type GetTypeCallback(
        string typeName,
        bool throwOnError,
        bool ignoreCase
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: This is used by the ShellMainCore method when it encounters an
    //       argument it cannot handle.
    //
    [ObjectId("dad9e62e-eeeb-4043-b474-4a21955f369a")]
    public delegate ReturnCode ShellArgumentCallback(
        Interpreter interpreter,
        IInteractiveHost interactiveHost,
        IClientData clientData,
        int count,
        string arg,
        ref IList<string> argv,
        ref Result result
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: This is used by the Debugger class when it needs to break into the
    //       interactive loop.
    //
    [ObjectId("5d85f38d-bc7b-4e30-9fa3-34eb64e3ada2")]
    public delegate ReturnCode InteractiveLoopCallback(
        Interpreter interpreter,
        bool debug,
        IEnumerable<string> args,
        ReturnCode code,
        BreakpointType breakpointType,
        string breakpointName,
        IToken token,
        ITraceInfo traceInfo,
        EngineFlags engineFlags,
        SubstitutionFlags substitutionFlags,
        EventFlags eventFlags,
        ExpressionFlags expressionFlags,
        HeaderFlags headerFlags,
        IClientData clientData,
        ArgumentList arguments,
        ref bool exit,
        ref Result result
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: Used by the script binder to implement dynamic type
    //       conversions from string (always in the context of an
    //       interpreter).
    //
    [ObjectId("c9a205cc-d60f-4e67-b5ef-309aebf1b46c")]
    public delegate ReturnCode ChangeTypeCallback(
        Interpreter interpreter, // TODO: Change this to use the IInterpreter type.
        Type type,
        string text,
        OptionDictionary options,
        CultureInfo cultureInfo,
        IClientData clientData,
        ref object value,
        ref Result error
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    [ObjectId("817fb51d-ac00-411d-b921-ab9688e68a1b")]
    public delegate ReturnCode ToStringCallback(
        Interpreter interpreter, // TODO: Change this to use the IInterpreter type.
        Type type,
        object value,
        OptionDictionary options,
        CultureInfo cultureInfo,
        IClientData clientData,
        ref string text,
        ref Result error
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: This is used to check if a character is a member of some
    //       subset of Unicode categories.
    //
    [ObjectId("45d162bb-0243-40c4-ba13-66ed4ae6c3a9")]
    public delegate bool CharIsCallback(char character);

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: This is used by the EventManager to provide a means of
    //       fetching the current DateTime (which may be "virtual").
    //
    [ObjectId("a495ae22-6f78-483c-a66a-39f084c76021")]
    public delegate DateTime DateTimeNowCallback();

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: This is used by the [clock] command machinery to handle
    //       Tcl format string compatibility.
    //
    [ObjectId("4b36c510-9db2-4177-86f5-3b990b59f299")]
    public delegate string ClockTransformCallback(IClockData clockData);

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: This is used by the StringList class to perform a transform
    //       on each element added to the newly created list.
    //
    [ObjectId("eecb9222-2372-4b29-bb03-de84545e5181")]
    public delegate string StringTransformCallback(string value);

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: These are used by the file command to get or set the
    //       created, modified, or last access time for a given file.
    //
    [ObjectId("a968cc80-afb9-4d19-90a0-66c55b5ee2c1")]
    public delegate DateTime GetFileDateTimeCallback(string path);

    [ObjectId("4ad448b1-59be-4ac1-afad-53bf9b8fbbb2")]
    public delegate void SetFileDateTimeCallback(string path, DateTime dateTime);

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: This is used by the package index search routine to notify
    //       the caller of a newly found package index.
    //
    [ObjectId("d5f45c92-d910-4e51-8415-01842a78b57d")]
    public delegate ReturnCode PackageIndexCallback(
        Interpreter interpreter, // TODO: Change this to use the IInterpreter type.
        string fileName,
        bool host,
        bool resolve,
        bool refresh,
        ref Result error
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    [ObjectId("890b5534-11fe-42ab-8549-fd823257115b")]
    public delegate IHost NewHostCallback(IHostData hostData);

    ///////////////////////////////////////////////////////////////////////////////////////////////

    [ObjectId("60472558-0049-40ba-9c08-5e3ac01d5acb")]
    public delegate void WriteLineCallback();

    ///////////////////////////////////////////////////////////////////////////////////////////////

    [ObjectId("d8feecac-4278-4070-8819-4bd5feb4dc79")]
    public delegate void WriteCharCallback(
        char value
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    [ObjectId("bdf5896b-81ee-48c4-93a1-b8a8c58d9027")]
    public delegate void WriteStringCallback(
        string value
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: Generic script evaluation callback (see also
    //       IAsynchronousContext interface).
    //
    [ObjectId("96804ca1-fad9-4e5e-beab-0e4c78c29044")]
    public delegate void AsynchronousCallback(
        IAsynchronousContext context
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: Generic interpreter asynchronous event callback.
    //
    [ObjectId("8e5cab46-9887-49e4-8746-39330df59c34")]
    public delegate ReturnCode EventCallback(
        Interpreter interpreter, // TODO: Change this to use the IInterpreter type.
        IClientData clientData,
        ref Result result
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: Interpreter command execution callback (see also
    //       "IExecute" interface).
    //
    [ObjectId("f566bc2c-242c-4a0b-a556-605f6b4b3833")]
    public delegate ReturnCode ExecuteCallback(
        Interpreter interpreter, // TODO: Change this to use the IInterpreter type.
        IClientData clientData,
        ArgumentList arguments,
        ref Result result
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

    //
    // NOTE: Interpreter variable trace callback.
    //
    [ObjectId("87929871-aaea-44b9-b6e1-fb6fdc9afaf1")]
    public delegate ReturnCode TraceCallback(
        BreakpointType breakpointType,
        Interpreter interpreter, // TODO: Change this to use the IInterpreter type.
        ITraceInfo traceInfo,
        ref Result result
    );

    ///////////////////////////////////////////////////////////////////////////////////////////////

#if NOTIFY || NOTIFY_OBJECT
    //
    // NOTE: Interpreter notification callback (see also
    //       "INotify" interface).
    //
    [ObjectId("2b4f0432-8412-4dde-afcc-8e556c37f5c2")]
    public delegate ReturnCode NotifyCallback(
        Interpreter interpreter, // TODO: Change this to use the IInterpreter type.
        IScriptEventArgs eventArgs,
        IClientData clientData,
        ArgumentList arguments,
        ref Result result
    );
#endif
}

