/*
 * Utility.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Reflection;
using System.Resources;
using System.Security.Cryptography.X509Certificates;

#if XML
using System.Xml;
#endif

using Eagle._Attributes;
using Eagle._Components.Private;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Eagle._Components.Public
{
    [ObjectId("702cb2b3-5e60-4f90-b5af-df09c236ef51")]
    public static class Utility
    {
        #region External Use Only Helper Methods
        //
        // WARNING: The returned DateTime value may be virtualized (i.e. it may
        //          not reflect the actual current date and time).
        //
        public static DateTime GetNow() /* EXTERNAL USE ONLY */
        {
            return TimeOps.GetNow();
        }

        ///////////////////////////////////////////////////////////////////////

        //
        // WARNING: The returned DateTime value may be virtualized (i.e. it may
        //          not reflect the actual current date and time).
        //
        public static DateTime GetUtcNow() /* EXTERNAL USE ONLY */
        {
            return TimeOps.GetUtcNow();
        }

        ///////////////////////////////////////////////////////////////////////

#if SHELL && INTERACTIVE_COMMANDS
        public static StringList GetInteractiveCommandNames( /* EXTERNAL USE ONLY */
            string pattern,
            bool noCase
            )
        {
            return HelpOps.GetInteractiveCommandNames(pattern, noCase);
        }

        ///////////////////////////////////////////////////////////////////////

        public static StringPair GetInteractiveCommandHelp( /* EXTERNAL USE ONLY */
            string name
            )
        {
            return HelpOps.GetInteractiveCommandHelp(name);
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        public static string BuildCommandLine( /* EXTERNAL USE ONLY */
            IEnumerable<string> args,
            bool quoteAll
            )
        {
            return RuntimeOps.BuildCommandLine(args, quoteAll);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string PopFirstArgument( /* EXTERNAL USE ONLY */
            ref IList<string> args
            )
        {
            return GenericOps<string>.PopFirstArgument(ref args);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string PopLastArgument( /* EXTERNAL USE ONLY */
            ref IList<string> args
            )
        {
            return GenericOps<string>.PopLastArgument(ref args);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool MatchSwitch( /* EXTERNAL USE ONLY */
            string text,
            string @switch
            )
        {
            return StringOps.MatchSwitch(text, @switch);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool HasFlags( /* EXTERNAL USE ONLY */
            BreakpointType flags,
            BreakpointType hasFlags,
            bool all
            )
        {
            return FlagOps.HasFlags(flags, hasFlags, all);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool HasFlags( /* EXTERNAL USE ONLY */
            CommandFlags flags,
            CommandFlags hasFlags,
            bool all
            )
        {
            return FlagOps.HasFlags(flags, hasFlags, all);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool HasFlags( /* EXTERNAL USE ONLY */
            CreateFlags flags,
            CreateFlags hasFlags,
            bool all
            )
        {
            return FlagOps.HasFlags(flags, hasFlags, all);
        }

        ///////////////////////////////////////////////////////////////////////

#if NOTIFY || NOTIFY_OBJECT
        public static bool HasFlags( /* EXTERNAL USE ONLY */
            NotifyFlags flags,
            NotifyFlags hasFlags,
            bool all
            )
        {
            return FlagOps.HasFlags(flags, hasFlags, all);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool HasFlags( /* EXTERNAL USE ONLY */
            NotifyType flags,
            NotifyType hasFlags,
            bool all
            )
        {
            return FlagOps.HasFlags(flags, hasFlags, all);
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        public static bool HasFlags( /* EXTERNAL USE ONLY */
            PluginFlags flags,
            PluginFlags hasFlags,
            bool all
            )
        {
            return FlagOps.HasFlags(flags, hasFlags, all);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool HasFlags( /* EXTERNAL USE ONLY */
            PolicyFlags flags,
            PolicyFlags hasFlags,
            bool all
            )
        {
            return FlagOps.HasFlags(flags, hasFlags, all);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool HasFlags( /* EXTERNAL USE ONLY */
            ScriptFlags flags,
            ScriptFlags hasFlags,
            bool all
            )
        {
            return FlagOps.HasFlags(flags, hasFlags, all);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool ShouldTraceToHost( /* EXTERNAL USE ONLY */
            Interpreter interpreter
            )
        {
            return DebugOps.SafeGetTraceToHost(interpreter, false);
        }

        ///////////////////////////////////////////////////////////////////////

        public static object Identity( /* EXTERNAL USE ONLY */
            object arg
            )
        {
            return MarshalOps.Identity(arg);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Assembly FindAssemblyInAppDomain( /* EXTERNAL USE ONLY */
            AppDomain appDomain,
            string name,
            Version version,
            byte[] publicKeyToken,
            ref Result error
            )
        {
            return AssemblyOps.FindInAppDomain(
                appDomain, name, version, publicKeyToken, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static InterpreterDictionary GetInterpreters() /* EXTERNAL USE ONLY */
        {
            return GlobalState.GetInterpreters();
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetPackageName( /* EXTERNAL USE ONLY */
            bool noCase
            )
        {
            return GlobalState.GetPackageName(noCase);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetPackageName( /* EXTERNAL USE ONLY */
            string prefix,
            string suffix,
            bool noCase
            )
        {
            return GlobalState.GetPackageName(prefix, suffix, noCase);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Version GetPackageVersion() /* EXTERNAL USE ONLY */
        {
            return GlobalState.GetPackageVersion();
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetPackagePath( /* EXTERNAL USE ONLY */
            Assembly assembly,
            string name,
            Version version,
            bool root
            )
        {
            return GlobalState.GetPackagePath(assembly, name, version, root);
        }

        ///////////////////////////////////////////////////////////////////////

        public static void SetBasePath( /* EXTERNAL USE ONLY */
            string basePath,
            bool refresh
            )
        {
            GlobalState.SetBasePath(basePath, refresh);
        }

        ///////////////////////////////////////////////////////////////////////

        public static void SetLibraryPath( /* EXTERNAL USE ONLY */
            string libraryPath,
            bool refresh
            )
        {
            GlobalState.SetLibraryPath(libraryPath, refresh);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ScriptTypeToFileName( /* EXTERNAL USE ONLY */
            string type,
            bool strict
            )
        {
            return FormatOps.ScriptTypeToFileName(type, strict);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool SetVariableUndefined( /* EXTERNAL USE ONLY */
            IVariable variable,
            bool undefined
            )
        {
            return EntityOps.SetUndefined(variable, undefined);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool SetVariableDirty( /* EXTERNAL USE ONLY */
            IVariable variable,
            bool dirty
            )
        {
            return EntityOps.SetDirty(variable, dirty);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatBreakpoint( /* EXTERNAL USE ONLY */
            BreakpointType breakpointType
            )
        {
            return FormatOps.Breakpoint(breakpointType);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatErrorVariableName( /* EXTERNAL USE ONLY */
            string varName,
            string varIndex
            )
        {
            return FormatOps.ErrorVariableName(varName, varIndex);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetBinaryPath() /* EXTERNAL USE ONLY */
        {
            return GlobalState.GetBinaryPath();
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetExecutableName() /* EXTERNAL USE ONLY */
        {
            return PathOps.GetExecutableName();
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode GetAssemblyCertificate( /* EXTERNAL USE ONLY */
            Assembly assembly,
            ref X509Certificate certificate,
            ref Result error
            )
        {
            return AssemblyOps.GetCertificate(
                assembly, ref certificate, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode GetAssemblyCertificate2( /* EXTERNAL USE ONLY */
            Assembly assembly,
            bool strict,
            ref X509Certificate2 certificate,
            ref Result error
            )
        {
            return AssemblyOps.GetCertificate2(
                assembly, strict, ref certificate, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Guid GetObjectId( /* EXTERNAL USE ONLY */
            MemberInfo memberInfo
            )
        {
            return AttributeOps.GetObjectId(memberInfo);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Guid GetObjectId( /* EXTERNAL USE ONLY */
            object @object
            )
        {
            return AttributeOps.GetObjectId(@object);
        }

        ///////////////////////////////////////////////////////////////////////

        public static CommandFlags GetCommandFlags( /* EXTERNAL USE ONLY */
            MemberInfo memberInfo
            )
        {
            return AttributeOps.GetCommandFlags(memberInfo);
        }

        ///////////////////////////////////////////////////////////////////////

        public static CommandFlags GetCommandFlags( /* EXTERNAL USE ONLY */
            object @object
            )
        {
            return AttributeOps.GetCommandFlags(@object);
        }

        ///////////////////////////////////////////////////////////////////////

        public static FunctionFlags GetFunctionFlags( /* EXTERNAL USE ONLY */
            MemberInfo memberInfo
            )
        {
            return AttributeOps.GetFunctionFlags(memberInfo);
        }

        ///////////////////////////////////////////////////////////////////////

        public static FunctionFlags GetFunctionFlags( /* EXTERNAL USE ONLY */
            object @object
            )
        {
            return AttributeOps.GetFunctionFlags(@object);
        }

        ///////////////////////////////////////////////////////////////////////

        public static PluginFlags GetPluginFlags( /* EXTERNAL USE ONLY */
            MemberInfo memberInfo
            )
        {
            return AttributeOps.GetPluginFlags(memberInfo);
        }

        ///////////////////////////////////////////////////////////////////////

        public static PluginFlags GetPluginFlags( /* EXTERNAL USE ONLY */
            object @object
            )
        {
            return AttributeOps.GetPluginFlags(@object);
        }

        ///////////////////////////////////////////////////////////////////////

        public static NotifyFlags GetNotifyFlags( /* EXTERNAL USE ONLY */
            MemberInfo memberInfo
            )
        {
            return AttributeOps.GetNotifyFlags(memberInfo);
        }

        ///////////////////////////////////////////////////////////////////////

        public static NotifyType GetNotifyTypes( /* EXTERNAL USE ONLY */
            MemberInfo memberInfo
            )
        {
            return AttributeOps.GetNotifyTypes(memberInfo);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetAssemblyConfiguration( /* EXTERNAL USE ONLY */
            Assembly assembly
            )
        {
            return AttributeOps.GetAssemblyConfiguration(assembly);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetAssemblyTitle( /* EXTERNAL USE ONLY */
            Assembly assembly
            )
        {
            return AttributeOps.GetAssemblyTitle(assembly);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetAssemblyDescription( /* EXTERNAL USE ONLY */
            Assembly assembly
            )
        {
            return AttributeOps.GetAssemblyDescription(assembly);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Uri GetAssemblyUri( /* EXTERNAL USE ONLY */
            Assembly assembly
            )
        {
            return AttributeOps.GetAssemblyUri(assembly);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetEnvironmentVariable( /* EXTERNAL USE ONLY */
            string variable
            )
        {
            return CommonOps.GetEnvironmentVariable(variable);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetEnvironmentVariable( /* EXTERNAL USE ONLY */
            string variable,
            bool configuration,
            bool expand
            )
        {
            return GlobalState.GetEnvironmentVariable(
                variable, configuration, expand);
        }

        ///////////////////////////////////////////////////////////////////////

        public static void SetEnvironmentVariable( /* EXTERNAL USE ONLY */
            string variable,
            string value
            )
        {
            GlobalState.SetEnvironmentVariable(variable, value);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ExpandEnvironmentVariables( /* EXTERNAL USE ONLY */
            string name
            )
        {
            return CommonOps.ExpandEnvironmentVariables(name);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetStringFromObject( /* EXTERNAL USE ONLY */
            object @object
            )
        {
            return RuntimeOps.GetStringFromObject(@object);
        }

        ///////////////////////////////////////////////////////////////////////

        public static TraceListener NewDefaultTraceListener( /* EXTERNAL USE ONLY */
            bool console
            )
        {
            return DebugOps.NewDefaultTraceListener(console);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode AddTraceListener( /* EXTERNAL USE ONLY */
            TraceListener listener,
            bool debug
            )
        {
            return DebugOps.AddTraceListener(listener, debug);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode RemoveTraceListener( /* EXTERNAL USE ONLY */
            TraceListener listener,
            bool debug
            )
        {
            return DebugOps.RemoveTraceListener(listener, debug);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool IsRemoteUri( /* EXTERNAL USE ONLY */
            string value
            ) 
        {
            return PathOps.IsRemoteUri(value);
        }

        ///////////////////////////////////////////////////////////////////////

        public static object TryParseEnum( /* EXTERNAL USE ONLY */
            Type enumType,
            string value,
            bool noCase
            )
        {
            return EnumOps.TryParseEnum(enumType, value, noCase);
        }

        ///////////////////////////////////////////////////////////////////////

        public static object TryParseEnum( /* EXTERNAL USE ONLY */
            Type enumType,
            string value,
            bool noCase,
            ref Result error
            )
        {
            return EnumOps.TryParseEnum(enumType, value, noCase, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static object TryParseFlagsEnum( /* EXTERNAL USE ONLY */
            Interpreter interpreter,
            Type enumType,
            string oldValue,
            string newValue,
            CultureInfo cultureInfo,
            bool noCase,
            ref Result error
            )
        {
            return EnumOps.TryParseFlagsEnum(
                interpreter, enumType, oldValue, newValue, cultureInfo,
                noCase, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ListToEnglish( /* EXTERNAL USE ONLY */
            IList<string> list,
            string prefix,
            string separator,
            string suffix
            )
        {
            return GenericOps<string>.ListToEnglish(
                list, prefix, separator, suffix);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ListToEnglish( /* EXTERNAL USE ONLY */
            IList<string> list,
            string prefix,
            string separator,
            string suffix,
            string valuePrefix,
            string valueSuffix
            )
        {
            return GenericOps<string>.ListToEnglish(
                list, prefix, separator, suffix, valuePrefix, valueSuffix);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ListToEnglish( /* EXTERNAL USE ONLY */
            IDictionary<string, string> list,
            string prefix,
            string separator,
            string suffix
            )
        {
            return GenericOps<string>.ListToEnglish(
                list, prefix, separator, suffix);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode TryDisposeObject( /* EXTERNAL USE ONLY */
            object @object,
            ref Result error
            )
        {
            return RuntimeOps.TryDisposeObject(@object, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode TryDisposeObject( /* EXTERNAL USE ONLY */
            object @object,
            ref bool dispose,
            ref Result error,
            ref Exception exception
            )
        {
            return RuntimeOps.TryDisposeObject(
                @object, ref dispose, ref error, ref exception);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string MakeStringId( /* EXTERNAL USE ONLY */
            string prefix,
            long id
            )
        {
            return FormatOps.Id(prefix, null, id);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode ParseHexavigesimal( /* EXTERNAL USE ONLY */
            string text,
            ref long value,
            ref Result error
            )
        {
            if (!String.IsNullOrEmpty(text) &&
                (Parser.ParseHexavigesimal(text, 0, text.Length,
                    ref value) == text.Length))
            {
                return ReturnCode.Ok;
            }

            error = String.Format(
                "expected hexavigesimal wide integer but got \"{0}\"",
                text);

            return ReturnCode.Error;
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatHexavigesimal( /* EXTERNAL USE ONLY */
            ulong value,
            byte width
            )
        {
            return FormatOps.Hexavigesimal(value, width);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool IsBase26( /* EXTERNAL USE ONLY */
            string text
            )
        {
            return StringOps.IsBase26(text);
        }

        ///////////////////////////////////////////////////////////////////////

        public static byte[] FromBase26String( /* EXTERNAL USE ONLY */
            string value
            )
        {
            return StringOps.FromBase26String(value);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ToBase26String( /* EXTERNAL USE ONLY */
            byte[] array,
            Base26FormattingOption options
            )
        {
            return StringOps.ToBase26String(array, options);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatPackageDirectory( /* EXTERNAL USE ONLY */
            string name,
            Version version,
            bool full
            )
        {
            return FormatOps.PackageDirectory(name, version, full);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatPluginName( /* EXTERNAL USE ONLY */
            string assemblyName,
            string typeName
            )
        {
            return FormatOps.PluginName(assemblyName, typeName);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatPluginAbout( /* EXTERNAL USE ONLY */
            IPlugin plugin,
            bool full
            )
        {
            return FormatOps.PluginAbout(plugin, full);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatId( /* EXTERNAL USE ONLY */
            string prefix,
            string name,
            long id
            )
        {
            return FormatOps.Id(prefix, name, id);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatResult( /* EXTERNAL USE ONLY */
            ReturnCode code,
            Result result
            )
        {
            return RuntimeOps.FormatResult(code, result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatResult( /* EXTERNAL USE ONLY */
            ReturnCode code,
            Result result,
            int errorLine
            )
        {
            return RuntimeOps.FormatResult(code, result, errorLine);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool IsSuccess( /* EXTERNAL USE ONLY */
            ReturnCode code,
            bool exceptions
            )
        {
            return RuntimeOps.IsSuccess(code, exceptions);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode CustomOkCode( /* EXTERNAL USE ONLY */
            uint value
            )
        {
            return RuntimeOps.CustomOkCode(value);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode CustomErrorCode( /* EXTERNAL USE ONLY */
            uint value
            )
        {
            return RuntimeOps.CustomErrorCode(value);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ExitCode SuccessExitCode() /* EXTERNAL USE ONLY */
        {
            return RuntimeOps.SuccessExitCode();
        }

        ///////////////////////////////////////////////////////////////////////

        public static ExitCode FailureExitCode() /* EXTERNAL USE ONLY */
        {
            return RuntimeOps.FailureExitCode();
        }

        ///////////////////////////////////////////////////////////////////////

        public static ExitCode ExceptionExitCode() /* EXTERNAL USE ONLY */
        {
            return RuntimeOps.ExceptionExitCode();
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode ExitCodeToReturnCode( /* EXTERNAL USE ONLY */
            ExitCode exitCode
            )
        {
            return RuntimeOps.ExitCodeToReturnCode(exitCode);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ExitCode ReturnCodeToExitCode( /* EXTERNAL USE ONLY */
            ReturnCode code,
            bool exceptions
            )
        {
            return RuntimeOps.ReturnCodeToExitCode(code, exceptions);
        }

        ///////////////////////////////////////////////////////////////////////

        public static uint FlipEndian( /* EXTERNAL USE ONLY */
            uint X
            )
        {
            return ConversionOps.FlipEndian(X);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ulong FlipEndian( /* EXTERNAL USE ONLY */
            ulong X
            )
        {
            return ConversionOps.FlipEndian(X);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ToHexadecimalString( /* EXTERNAL USE ONLY */
            byte[] array
            )
        {
            return ArrayOps.ToHexadecimalString(array);
        }

        ///////////////////////////////////////////////////////////////////////

#if XML && SERIALIZATION
        public static ReturnCode Serialize( /* EXTERNAL USE ONLY */
            object @object,
            Type type,
            XmlWriter writer,
            ref Result error
            )
        {
            return XmlOps.Serialize(@object, type, writer, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode Deserialize( /* EXTERNAL USE ONLY */
            Type type,
            XmlReader reader,
            ref object @object,
            ref Result error
            )
        {
            return XmlOps.Deserialize(type, reader, ref @object, ref error);
        }
#endif

        ///////////////////////////////////////////////////////////////////////

#if XML
        public static ReturnCode Validate( /* EXTERNAL USE ONLY */
            Assembly assembly,
            string resourceName,
            Uri namespaceUri,
            XmlDocument document,
            ref Result error
            )
        {
            return XmlOps.Validate(
                assembly, resourceName, namespaceUri, document, ref error);
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        public static OptionDictionary GetInvokeOptions( /* EXTERNAL USE ONLY */
            bool all
            )
        {
            return RuntimeOps.GetInvokeOptions(all);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode FixupReturnValue( /* EXTERNAL USE ONLY */
            Interpreter interpreter,  /* in */
            Type type,                /* in */
            ObjectFlags flags,        /* in */
            string objectName,        /* in */
            object value,             /* in */
            bool alias,               /* in */
            bool aliasAll,            /* in */
            ref Result result         /* out */
            )
        {
            return MarshalOps.FixupReturnValue(
                interpreter, type, flags, objectName, value,
                true, alias, aliasAll, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode FixupReturnValue( /* EXTERNAL USE ONLY */
            Interpreter interpreter,  /* in */
            IBinder binder,           /* in */
            CultureInfo cultureInfo,  /* in */
            Type type,                /* in */
            ObjectFlags flags,        /* in */
            OptionDictionary options, /* in */
            string objectName,        /* in */
            string interpName,        /* in */
            object value,             /* in */
            bool create,              /* in */
            bool dispose,             /* in */
            bool alias,               /* in */
            bool aliasAll,            /* in */
            bool toString,            /* in */
            ref Result result         /* out */
            )
        {
            return MarshalOps.FixupReturnValue(
                interpreter, binder, cultureInfo, type, flags,
                options, objectName, interpName, value, create,
                dispose, alias, aliasAll, toString, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static void ThrowFeatureNotSupported( /* EXTERNAL USE ONLY */
            IPlugin plugin,
            string name
            )
        {
            RuntimeOps.ThrowFeatureNotSupported(plugin, name);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode SubCommandPolicy( /* EXTERNAL USE ONLY */
            PolicyFlags flags,
            Type commandType,
            long commandToken,
            StringList subCommands,
            bool allowed,
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            return PolicyOps.SubCommandPolicy(
                flags, commandType, commandToken, subCommands,
                allowed, interpreter, clientData, arguments,
                ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode DirectoryPolicy( /* EXTERNAL USE ONLY */
            PolicyFlags flags,
            Type commandType,
            long commandToken,
            string fileName,
            PathDictionary<object> directories,
            bool allowed,
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            return PolicyOps.DirectoryPolicy(
                flags, commandType, commandToken, fileName,
                directories, allowed, interpreter, clientData,
                arguments, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode UriPolicy( /* EXTERNAL USE ONLY */
            PolicyFlags flags,
            Type commandType,
            long commandToken,
            Uri uri,
            UriDictionary<object> uris,
            bool allowed,
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            return PolicyOps.UriPolicy(
                flags, commandType, commandToken, uri, uris,
                allowed, interpreter, clientData, arguments,
                ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode CallbackPolicy( /* EXTERNAL USE ONLY */
            PolicyFlags flags,
            Type commandType,
            long commandToken,
            ICallback callback,
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            return PolicyOps.CallbackPolicy(
                flags, commandType, commandToken, callback,
                interpreter, clientData, arguments, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode ScriptPolicy( /* EXTERNAL USE ONLY */
            PolicyFlags flags,
            Type commandType,
            long commandToken,
            Interpreter policyInterpreter,
            string text,
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            return PolicyOps.ScriptPolicy(
                flags, commandType, commandToken, policyInterpreter,
                text, interpreter, clientData, arguments, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode TypePolicy( /* EXTERNAL USE ONLY */
            PolicyFlags flags,
            Type commandType,
            long commandToken,
            Type objectType,
            TypeList types,
            bool allowed,
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            return PolicyOps.TypePolicy(
                flags, commandType, commandToken, objectType,
                types, allowed, interpreter, clientData,
                arguments, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool IsSameFile( /* EXTERNAL USE ONLY */
            StringList paths,
            string path2
            )
        {
            foreach (string path1 in paths)
                if (PathOps.IsSameFile(path1, path2))
                    return true;

            return false;
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool IsSameFile( /* EXTERNAL USE ONLY */
            string path1,
            string path2
            )
        {
            return PathOps.IsSameFile(path1, path2);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string NormalizePath( /* EXTERNAL USE ONLY */
            string path
            )
        {
            return PathOps.ResolvePath(path);
        }

        ///////////////////////////////////////////////////////////////////////

#if !MONO
        public static ReturnCode VerifyPath( /* EXTERNAL USE ONLY */
            string path,
            FilePermission permissions,
            ref Result error
            )
        {
            return FileOps.VerifyPath(path, permissions, ref error);
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        public static int CompareFileNames( /* EXTERNAL USE ONLY */
            string fileNameA,
            string fileNameB
            )
        {
            return String.Compare(
                fileNameA, fileNameB, PathOps.ComparisonType);
        }

        ///////////////////////////////////////////////////////////////////////

        public static int CompareUserStrings( /* EXTERNAL USE ONLY */
            string strA,
            string strB,
            bool noCase
            )
        {
            return String.Compare(strA, strB, noCase ?
                StringOps.UserNoCaseStringComparisonType :
                StringOps.UserStringComparisonType);
        }

        ///////////////////////////////////////////////////////////////////////

        public static int CompareSystemStrings( /* EXTERNAL USE ONLY */
            string strA,
            string strB,
            bool noCase
            )
        {
            return String.Compare(strA, strB, noCase ?
                StringOps.SystemNoCaseStringComparisonType :
                StringOps.SystemStringComparisonType);
        }

        ///////////////////////////////////////////////////////////////////////

        public static long DefaultAttributeFlagsKey() /* EXTERNAL USE ONLY */
        {
            return AttributeFlags.NoKey;
        }

        ///////////////////////////////////////////////////////////////////////

        public static IDictionary<long, string> ParseAttributeFlags( /* EXTERNAL USE ONLY */
            string text,
            bool complex,
            bool space,
            bool sort,
            ref Result error
            )
        {
            return AttributeFlags.Parse(
                text, complex, space, sort, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatAttributeFlags( /* EXTERNAL USE ONLY */
            IDictionary<long, string> flags,
            bool legacy,
            bool compact,
            bool space,
            bool sort,
            ref Result error
            )
        {
            return AttributeFlags.Format(
                flags, legacy, compact, space, sort, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool VerifyAttributeFlags( /* EXTERNAL USE ONLY */
            string text,
            bool complex,
            bool space,
            ref Result error
            )
        {
            return AttributeFlags.Verify(text, complex, space, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool HaveAttributeFlags( /* EXTERNAL USE ONLY */
            IDictionary<long, string> flags,
            long key,
            string haveFlags,
            bool all,
            bool strict
            )
        {
            return AttributeFlags.Have(flags, key, haveFlags, all, strict);
        }

        ///////////////////////////////////////////////////////////////////////

        public static IDictionary<long, string> ChangeAttributeFlags( /* EXTERNAL USE ONLY */
            IDictionary<long, string> flags,
            long key,
            string changeFlags,
            bool sort,
            ref Result error
            )
        {
            return AttributeFlags.Change(
                flags, key, changeFlags, sort, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Result BadValue( /* EXTERNAL USE ONLY */
            string adjective,
            string type,
            string value,
            IEnumerable<string> values,
            string prefix,
            string suffix
            )
        {
            return RuntimeOps.BadValue(
                adjective, type, value, values, prefix, suffix);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Result BadSubCommand( /* EXTERNAL USE ONLY */
            string adjective,
            string type,
            string subCommand,
            IEnsemble ensemble,
            string prefix,
            string suffix
            )
        {
            return RuntimeOps.BadSubCommand(
                adjective, type, subCommand, ensemble, prefix, suffix);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Result WrongNumberOfArguments( /* EXTERNAL USE ONLY */
            IIdentifier identifier,
            int count,
            ArgumentList arguments,
            string suffix
            )
        {
            return RuntimeOps.WrongNumberOfArguments(
                identifier, count, arguments, suffix);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode TryExecuteSubCommandFromEnsemble( /* EXTERNAL USE ONLY */
            IEnsemble ensemble,
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            bool strict,
            bool noCase,
            ref string name,
            ref bool tried,
            ref Result result
            )
        {
            return RuntimeOps.TryExecuteSubCommandFromEnsemble(
                ensemble, interpreter, clientData, arguments, strict, noCase,
                ref name, ref tried, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode PrepareStaticPlugin( /* EXTERNAL USE ONLY */
            IPlugin plugin,
            ref Result error
            )
        {
            return RuntimeOps.PrepareStaticPlugin(plugin, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode ExtractPolicyContextAndCommand( /* EXTERNAL USE ONLY */
            Interpreter interpreter,
            IClientData clientData,
            Type commandType,
            long commandToken,
            ref IPolicyContext policyContext,
            ref bool match,
            ref Result error
            )
        {
            return RuntimeOps.ExtractPolicyContextAndCommand(
                interpreter, clientData, commandType, commandToken,
                ref policyContext, ref match, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode ExtractPolicyContextAndScript( /* EXTERNAL USE ONLY */
            Interpreter interpreter,
            IClientData clientData,
            ref IPolicyContext policyContext,
            ref IScript script,
            ref Result error
            )
        {
            return RuntimeOps.ExtractPolicyContextAndScript(
                interpreter, clientData, ref policyContext, ref script,
                ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode ExtractPolicyContextAndFileName( /* EXTERNAL USE ONLY */
            Interpreter interpreter,
            IClientData clientData,
            ref IPolicyContext policyContext,
            ref string fileName,
            ref Result error
            )
        {
            return RuntimeOps.ExtractPolicyContextAndFileName(
                interpreter, clientData, ref policyContext, ref fileName,
                ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode ExtractPolicyContextAndText( /* EXTERNAL USE ONLY */
            Interpreter interpreter,
            IClientData clientData,
            ref IPolicyContext policyContext,
            ref string text,
            ref Result error
            )
        {
            return RuntimeOps.ExtractPolicyContextAndText(
                interpreter, clientData, ref policyContext, ref text,
                ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool DetectLibraryPath( /* EXTERNAL USE ONLY */
            Assembly assembly
            )
        {
            return GlobalState.DetectLibraryPath(assembly);
        }

        ///////////////////////////////////////////////////////////////////////

        public static Result CreateSynchronizedResult( /* EXTERNAL USE ONLY */
            string name
            )
        {
            return RuntimeOps.CreateSynchronizedResult(name);
        }

        ///////////////////////////////////////////////////////////////////////

        public static void CleanupSynchronizedResult( /* EXTERNAL USE ONLY */
            Result synchronizedResult
            )
        {
            RuntimeOps.CleanupSynchronizedResult(synchronizedResult);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool WaitSynchronizedResult( /* EXTERNAL USE ONLY */
            Result synchronizedResult,
            int milliseconds
            )
        {
            return RuntimeOps.WaitSynchronizedResult(
                synchronizedResult, milliseconds);
        }

        ///////////////////////////////////////////////////////////////////////

        public static void SetSynchronizedResult( /* EXTERNAL USE ONLY */
            Result synchronizedResult,
            ReturnCode code,
            Result result
            )
        {
            RuntimeOps.SetSynchronizedResult(synchronizedResult, code, result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode GetSynchronizedResult( /* EXTERNAL USE ONLY */
            Result synchronizedResult,
            ref ReturnCode code,
            ref Result result,
            ref Result error
            )
        {
            return RuntimeOps.GetSynchronizedResult(
                synchronizedResult, ref code, ref result, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

#if NETWORK
        public static bool IsSoftwareUpdateTrusted() /* EXTERNAL USE ONLY */
        {
            return SocketOps.IsSoftwareUpdateTrusted();
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode SetSoftwareUpdateTrusted( /* EXTERNAL USE ONLY */
            bool trusted,
            ref Result error
            )
        {
            return SocketOps.SetSoftwareUpdateTrusted(trusted, ref error);
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        public static bool IsDefaultAppDomain() /* EXTERNAL USE ONLY */
        {
            return RuntimeOps.IsDefaultAppDomain();
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool IsSameAppDomain( /* EXTERNAL USE ONLY */
            Interpreter interpreter
            )
        {
            return RuntimeOps.IsSameAppDomain(interpreter);
        }

        ///////////////////////////////////////////////////////////////////////

        public static int GetCurrentThreadId() /* EXTERNAL USE ONLY */
        {
            return GlobalState.GetCurrentThreadId();
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode InvokeDelegate( /* EXTERNAL USE ONLY */
            Interpreter interpreter,
            Delegate @delegate,
            ArgumentList arguments,
            ref Result result
            )
        {
            return RuntimeOps.InvokeDelegate(
                interpreter, @delegate, arguments, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode WriteViaIExecute( /* EXTERNAL USE ONLY */
            Interpreter interpreter,
            string commandName, /* NOTE: Almost always null, for [puts]. */
            string channelId,   /* NOTE: Almost always null, for "stdout". */
            string value,
            ref Result result
            )
        {
            return RuntimeOps.WriteViaIExecute(
                interpreter, commandName, channelId, value, ref result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatWrapOrNull( /* EXTERNAL USE ONLY */
            object value
            )
        {
            return FormatOps.WrapOrNull(value);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string FormatWrapOrNull( /* EXTERNAL USE ONLY */
            bool normalize,
            bool ellipsis,
            object value
            )
        {
            return FormatOps.WrapOrNull(normalize, ellipsis, value);
        }

        ///////////////////////////////////////////////////////////////////////

        public static bool ArrayEquals( /* EXTERNAL USE ONLY */
            byte[] array1,
            byte[] array2
            )
        {
            return RuntimeOps.ArrayEquals(array1, array2);
        }

        ///////////////////////////////////////////////////////////////////////

        public static void DebugTrace( /* EXTERNAL USE ONLY */
            string message,
            string category
            )
        {
            RuntimeOps.DebugTrace(message, category);
        }

        ///////////////////////////////////////////////////////////////////////

        public static void Complain( /* EXTERNAL USE ONLY */
            Interpreter interpreter,
            ReturnCode code,
            Result result
            )
        {
            DebugOps.Complain(interpreter, code, result);
        }

        ///////////////////////////////////////////////////////////////////////

        public static long NextId() /* EXTERNAL USE ONLY */
        {
            return GlobalState.NextId();
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetString( /* EXTERNAL USE ONLY */
            ResourceManager resourceManager,
            string name,
            CultureInfo cultureInfo,
            ref Result error
            )
        {
            return RuntimeOps.GetString(
                resourceManager, name, cultureInfo, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static string GetTclVersion() /* EXTERNAL USE ONLY */
        {
            return TclVars.VersionValue;
        }

        ///////////////////////////////////////////////////////////////////////

#if ISOLATED_PLUGINS
        public static bool IsPluginIsolated( /* EXTERNAL USE ONLY */
            IPlugin plugin
            )
        {
            return RuntimeOps.IsPluginIsolated(plugin);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode FixupOptions( /* EXTERNAL USE ONLY */
            IPlugin plugin,
            OptionDictionary options,
            bool strict,
            ref Result error
            )
        {
            return RuntimeOps.FixupOptions(plugin, options, strict, ref error);
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        //
        // HACK: This method needs to be public because external applications
        //       and plugins may set the environment variables we care about;
        //       however, there is no other way to notify this library about
        //       those changes (other than this method, that is).
        //
        public static void RefreshAutoPathList( /* EXTERNAL USE ONLY */
            bool verbose
            )
        {
            GlobalState.RefreshAutoPathList(verbose);
        }

        ///////////////////////////////////////////////////////////////////////

#if NETWORK
        public static ReturnCode DownloadData( /* EXTERNAL USE ONLY */
            Uri uri,
            bool trusted,
            ref byte[] bytes,
            ref Result error
            )
        {
            return SocketOps.DownloadData(uri, trusted, ref bytes, ref error);
        }

        ///////////////////////////////////////////////////////////////////////

        public static ReturnCode DownloadFile( /* EXTERNAL USE ONLY */
            Uri uri,
            string fileName,
            bool trusted,
            ref Result error
            )
        {
            return SocketOps.DownloadFile(uri, fileName, trusted, ref error);
        }
#endif
        #endregion
    }
}
