/*
 * ScriptThread.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using System.Collections.Generic;
using System.Threading;
using Eagle._Attributes;
using Eagle._Components.Public;
using Eagle._Components.Public.Delegates;

namespace Eagle._Interfaces.Public
{
    [ObjectId("a93e417d-5ad1-413e-bd26-feb58d04a0f0")]
    public interface IScriptThread : IGetInterpreter
    {
        ///////////////////////////////////////////////////////////////////////
        // OBJECT IDENTITY & AFFINITY
        ///////////////////////////////////////////////////////////////////////

        Thread Thread { get; }
        long Id { get; }
        string Name { get; }

        ///////////////////////////////////////////////////////////////////////
        // THREAD CREATION & SETUP
        ///////////////////////////////////////////////////////////////////////

        ThreadFlags ThreadFlags { get; }
        int MaxStackSize { get; }
        int Timeout { get; }

        ///////////////////////////////////////////////////////////////////////
        // INTERPRETER CREATION & SETUP
        ///////////////////////////////////////////////////////////////////////

        IEnumerable<string> Args { get; }
        IHost Host { get; }
        CreateFlags CreateFlags { get; }
        IScript Script { get; }
        string VarName { get; }

        ///////////////////////////////////////////////////////////////////////
        // THREAD CREATION READ-ONLY PROPERTIES
        ///////////////////////////////////////////////////////////////////////

        bool UserInterface { get; }
        bool IsBackground { get; }
        bool UseSelf { get; }

        ///////////////////////////////////////////////////////////////////////
        // ERROR HANDLING READ-ONLY PROPERTIES
        ///////////////////////////////////////////////////////////////////////

        bool Quiet { get; }
        bool NoBackgroundError { get; }

        ///////////////////////////////////////////////////////////////////////
        // EVENT HANDLING READ-ONLY PROPERTIES
        ///////////////////////////////////////////////////////////////////////

        bool NoCancel { get; }
        bool StopOnError { get; }
        bool ErrorOnEmpty { get; }
        bool NoComplain { get; }

        ///////////////////////////////////////////////////////////////////////
        // DIAGNOSTIC READ-WRITE PROPERTIES
        ///////////////////////////////////////////////////////////////////////

        bool Debug { get; set; }
        ReturnCode ReturnCode { get; set; }
        Result Result { get; set; }

        ///////////////////////////////////////////////////////////////////////
        // THREAD STATE READ-ONLY PROPERTIES
        ///////////////////////////////////////////////////////////////////////

        bool IsAlive { get; }
        bool IsBusy { get; }

        ///////////////////////////////////////////////////////////////////////
        // THREAD STATE METHODS
        ///////////////////////////////////////////////////////////////////////

        bool Start();
        bool Stop(bool force);

        ///////////////////////////////////////////////////////////////////////
        // CLR OBJECT INTEGRATION METHODS
        ///////////////////////////////////////////////////////////////////////

        ReturnCode AddObject(object value);
        ReturnCode AddObject(object value, ref Result result);
        ReturnCode AddObject(object value, bool alias, ref Result result);
        ReturnCode AddObject(object value, bool alias, bool aliasAll,
            ref Result result);
        ReturnCode AddObject(ObjectFlags objectFlags, object value, bool alias,
            bool aliasAll, ref Result result);
        ReturnCode AddObject(string objectName, ObjectFlags objectFlags,
            object value, bool alias, bool aliasAll, ref Result result);

        ///////////////////////////////////////////////////////////////////////
        // SYNCHRONOUS WAIT METHODS
        ///////////////////////////////////////////////////////////////////////

        bool WaitForEnd();
        bool WaitForEnd(int milliseconds);

        bool WaitForEmpty();
        bool WaitForEmpty(int milliseconds);

        bool WaitForEvent();
        bool WaitForEvent(int milliseconds);

        ///////////////////////////////////////////////////////////////////////
        // ASYNCHRONOUS QUEUEING METHODS (VIA EVENT MANAGER)
        ///////////////////////////////////////////////////////////////////////

        bool Queue(EventCallback callback, IClientData clientData);
        bool Queue(DateTime dateTime, EventCallback callback,
            IClientData clientData);

        ///////////////////////////////////////////////////////////////////////
        // ASYNCHRONOUS QUEUEING METHODS (VIA ENGINE)
        ///////////////////////////////////////////////////////////////////////

        bool Queue(string text);
        bool Queue(DateTime dateTime, string text);
        bool Queue(string text, AsynchronousCallback callback,
            IClientData clientData);

        ///////////////////////////////////////////////////////////////////////
        // SYNCHRONOUS QUEUEING METHODS (VIA EVENT MANAGER AND/OR ENGINE)
        ///////////////////////////////////////////////////////////////////////

        ReturnCode Send(string text, ref Result result);
        ReturnCode Send(string text, bool useEngine, ref Result result);
        ReturnCode Send(string text, int milliseconds, bool useEngine,
            ref Result result);

        ///////////////////////////////////////////////////////////////////////
        // ASYNCHRONOUS SIGNALING METHODS
        ///////////////////////////////////////////////////////////////////////

        bool Signal(string value);
        bool WakeUp();

        ///////////////////////////////////////////////////////////////////////
        // ASYNCHRONOUS SCRIPT CANCELLATION METHODS
        ///////////////////////////////////////////////////////////////////////

        bool Cancel(bool unwind);
        bool ResetCancel(bool force);
    }
}
